function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var reactKonva = require('react-konva');
var React = require('react');
var React__default = _interopDefault(React);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var Z_INDEX = 0;
var PORTAL_LAYER_ID = 'react-konva-portals';
var ERROR_MESSAGE = 'You should wrap your root component with <Stage /> from "react-konva-portal" in order for Portals to work';
function useForceRender() {
  var _useReducer = React.useReducer(function (i) {
    return i + 1;
  }, 0),
      forceRender = _useReducer[1];

  return forceRender;
}
function isStageContext(val) {
  return val && typeof val === 'object' && typeof val.mount === 'function' && typeof val.update === 'function' && typeof val.unmount === 'function' && typeof val.addManager === 'function' && typeof val.removeManager === 'function';
}
function assertStageContext(val) {
  if (!isStageContext(val)) throw new Error(ERROR_MESSAGE);
}
function zIndexComparator(a, b) {
  var _a$zIndex = a.zIndex,
      zIndexA = _a$zIndex === void 0 ? Z_INDEX : _a$zIndex;
  var _b$zIndex = b.zIndex,
      zIndexB = _b$zIndex === void 0 ? Z_INDEX : _b$zIndex;
  if (zIndexA === zIndexB) return 0;
  return zIndexA > zIndexB ? -1 : 1;
}
function mountCmd(id, key, zIndex, children) {
  return {
    type: 'mount',
    id: id,
    key: key,
    zIndex: zIndex,
    children: children
  };
}
function updateCmd(id, key, zIndex, children) {
  return {
    type: 'update',
    id: id,
    key: key,
    zIndex: zIndex,
    children: children
  };
}
function unmountCmd(id, key) {
  return {
    type: 'unmount',
    key: key,
    id: id
  };
}
function notUnmountByKey(key) {
  return function (item) {
    return item.key === key && item.type !== 'unmount';
  };
}
function warnIfDev() {
  if (process.env.NODE_ENV === 'development') {
    var _console;

    (_console = console).warn.apply(_console, arguments);
  }
}

function ManagerComponent(_, ref) {
  var portalsRef = React.useRef([]);
  var portalsByIdRef = React.useRef({});
  var unmountedRef = React.useRef(false);
  var forceRender = useForceRender();
  React.useEffect(function () {
    return function () {
      unmountedRef.current = true;
    };
  }, []);
  React.useImperativeHandle(ref, function () {
    function mount(cmd) {
      portalsByIdRef.current[cmd.key] = cmd;
      portalsRef.current.push(_extends({}, cmd));
      portalsRef.current = portalsRef.current.sort(zIndexComparator);
    }

    function update(cmd) {
      var curr = portalsByIdRef.current[cmd.key] || {};
      var resort = curr.zIndex !== cmd.zIndex;
      curr.key = cmd.key;
      curr.zIndex = cmd.zIndex;
      curr.children = cmd.children;
      portalsByIdRef.current[cmd.key] = curr;
      if (resort) portalsRef.current = portalsRef.current.sort(zIndexComparator);
    }

    function unmount(cmd) {
      delete portalsByIdRef.current[cmd.key];
      portalsRef.current = portalsRef.current.filter(function (item) {
        return item.key !== cmd.key;
      });
    }

    var handlers = {
      mount: mount,
      unmount: unmount,
      update: update
    };

    function handleCommand(cmd) {
      handlers[cmd.type].call(null, cmd);
    }

    function commitUpdate() {
      if (!unmountedRef.current) forceRender();
    }

    return {
      handle: function handle(input) {
        if (Array.isArray(input)) input.forEach(handleCommand);else handleCommand(input);
        commitUpdate();
      }
    };
  }, [forceRender]);
  return portalsRef.current.map(function (_ref) {
    var key = _ref.key,
        children = _ref.children;
    return React__default.createElement(React__default.Fragment, {
      key: key
    }, children);
  });
}

var PortalManager = React.forwardRef(ManagerComponent);

var StageContext = React.createContext(null);
function useStageContext() {
  var value = React.useContext(StageContext);
  React.useLayoutEffect(function () {
    return assertStageContext(value);
  }, [value]);
  return value;
}

function Provider(_ref) {
  var mount = _ref.mount,
      unmount = _ref.unmount,
      update = _ref.update,
      children = _ref.children,
      addManager = _ref.addManager,
      removeManager = _ref.removeManager;
  var value = React.useMemo(function () {
    return {
      mount: mount,
      unmount: unmount,
      update: update,
      addManager: addManager,
      removeManager: removeManager
    };
  }, [mount, unmount, update, addManager, removeManager]);
  return React__default.createElement(StageContext.Provider, {
    value: value
  }, children);
}

function createPortalContainer(Component, auditName) {
  function PortalContainer(props, ref) {
    var id = props.id,
        children = props.children;
    var konvaRef = React.useRef(null);
    var managerRef = React.useRef(null);
    var stage = useStageContext();
    var Konva = Component;
    React.useImperativeHandle(ref, function () {
      return konvaRef.current;
    }, [konvaRef.current]);
    React.useEffect(function () {
      if (id == null) {
        warnIfDev("Portals are not unavailable for " + auditName + " without id.");
      }
    }, [id, auditName]);
    React.useEffect(function () {
      if (id != null && managerRef.current) stage === null || stage === void 0 ? void 0 : stage.addManager(auditName, id, managerRef.current);
      return function () {
        if (id != null) stage === null || stage === void 0 ? void 0 : stage.removeManager(auditName, id);
      };
    }, [id, stage, auditName]);
    return React__default.createElement(Konva, Object.assign({}, props, {
      id: id,
      ref: konvaRef
    }), children, React__default.createElement(PortalManager, {
      id: id,
      ref: managerRef,
      auditName: auditName,
      key: id
    }));
  }

  return React.forwardRef(PortalContainer);
}

var PortalState;

(function (PortalState) {
  PortalState[PortalState["NONE"] = 0] = "NONE";
  PortalState[PortalState["WILL_MOUNT"] = 1] = "WILL_MOUNT";
  PortalState[PortalState["DID_MOUNT"] = 2] = "DID_MOUNT";
  PortalState[PortalState["WILL_UNMOUNT"] = 3] = "WILL_UNMOUNT";
})(PortalState || (PortalState = {}));

function Teleport(props) {
  var children = props.children,
      _props$containerId = props.containerId,
      containerId = _props$containerId === void 0 ? PORTAL_LAYER_ID : _props$containerId,
      _props$zIndex = props.zIndex,
      zIndex = _props$zIndex === void 0 ? Z_INDEX : _props$zIndex;
  var stage = useStageContext();
  var keyRef = React.useRef(0);
  var phaseRef = React.useRef(PortalState.NONE);
  React.useEffect(function () {
    return function () {
      phaseRef.current = PortalState.WILL_UNMOUNT;
    };
  }, []);
  var mountAsync = React.useCallback(function () {
    try {
      return Promise.resolve(Promise.resolve()).then(function () {
        keyRef.current = stage === null || stage === void 0 ? void 0 : stage.mount(containerId, zIndex, children);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }, [stage, containerId, zIndex, children]);
  React.useEffect(function () {
    if (phaseRef.current === PortalState.NONE) {
      phaseRef.current = PortalState.WILL_MOUNT;
      mountAsync().then(function () {
        phaseRef.current = PortalState.DID_MOUNT;
      });
    }

    if (phaseRef.current === PortalState.DID_MOUNT) {
      stage === null || stage === void 0 ? void 0 : stage.update(containerId, keyRef.current, zIndex, children);
    }

    return function () {
      if (phaseRef.current === PortalState.WILL_UNMOUNT) {
        stage === null || stage === void 0 ? void 0 : stage.unmount(containerId, keyRef.current);
      }
    };
  }, [mountAsync, stage, containerId, zIndex, children]);
  return null;
}

function Portal(_ref) {
  var containerId = _ref.containerId,
      props = _objectWithoutPropertiesLoose(_ref, ["containerId"]);

  return React__default.createElement(Teleport, Object.assign({}, props, {
    key: containerId,
    containerId: containerId
  }));
}

var PortalLayer = createPortalContainer(reactKonva.Layer, 'Layer');

function StageComponent(_ref, ref) {
  var children = _ref.children,
      portalLayerProps = _ref.portalLayerProps,
      props = _objectWithoutPropertiesLoose(_ref, ["children", "portalLayerProps"]);

  var seqRef = React.useRef(0);
  var stageRef = React.useRef(null);
  var queueRef = React.useRef({});
  var managersRef = React.useRef({});
  var deferRef = React.useRef(true);
  React.useImperativeHandle(ref, function () {
    return stageRef.current;
  }, [stageRef.current]);
  React.useEffect(function () {
    Object.keys(queueRef.current).forEach(function (id) {
      var _managersRef$current$;

      return (_managersRef$current$ = managersRef.current[id]) === null || _managersRef$current$ === void 0 ? void 0 : _managersRef$current$.handle(queueRef.current[id]);
    });
    deferRef.current = false;
    queueRef.current = {};
    return function () {
      deferRef.current = true;
    };
  }, []);
  var addManager = React.useCallback(function (auditName, id, manager) {
    if (managersRef.current[id]) {
      warnIfDev("Fond " + auditName + " with duplicate id \"" + id + "\".");
    }

    managersRef.current[id] = manager;
  }, []);
  var removeManager = React.useCallback(function (auditName, id) {
    if (!managersRef.current[id]) {
      warnIfDev("You are trying to remove unknown " + auditName + " instance with id \"" + id + "\".");
    }

    delete managersRef.current[id];
  }, []);
  var mount = React.useCallback(function (id, zIndex, children) {
    var key = ++seqRef.current;
    var managerReady = managersRef.current[id];
    if (managerReady && !deferRef.current) managerReady.handle(mountCmd(id, key, zIndex, children));else {
      var queue = queueRef.current[id] || [];
      queueRef.current[id] = queue;
      queue.push(mountCmd(id, key, zIndex, children));
    }
    return key;
  }, []);
  var update = React.useCallback(function (id, key, zIndex, children) {
    var managerReady = managersRef.current[id];
    if (managerReady && !deferRef.current) managerReady.handle(updateCmd(id, key, zIndex, children));else {
      var queue = queueRef.current[id] || [];
      queueRef.current[id] = queue;
      var index = queue.findIndex(notUnmountByKey(key));
      if (index === -1) queue.push(mountCmd(id, key, zIndex, children));else queue.splice(index, 1, mountCmd(id, key, zIndex, children));
    }
  }, []);
  var unmount = React.useCallback(function (id, key) {
    var managerReady = managersRef.current[id];
    if (managerReady && !deferRef.current) managerReady.handle(unmountCmd(id, key));else {
      var queue = queueRef.current[id] || [];
      queueRef.current[id] = queue;
      queue.push(unmountCmd(id, key));
    }
  }, []);
  React.useEffect(function () {
    return function () {
      Object.keys(queueRef.current).forEach(function (id) {
        var _managersRef$current$2;

        return (_managersRef$current$2 = managersRef.current[id]) === null || _managersRef$current$2 === void 0 ? void 0 : _managersRef$current$2.handle(queueRef.current[id]);
      });
      deferRef.current = false;
      queueRef.current = {};
    };
  }, []);
  return React__default.createElement(reactKonva.Stage, Object.assign({
    ref: stageRef
  }, props), React__default.createElement(Provider, {
    mount: mount,
    update: update,
    unmount: unmount,
    addManager: addManager,
    removeManager: removeManager
  }, children, React__default.createElement(PortalLayer, Object.assign({}, portalLayerProps, {
    id: PORTAL_LAYER_ID
  }))));
}

var PortalStage = React.forwardRef(StageComponent);

var Group = createPortalContainer(reactKonva.Group, 'Group');
var Layer = createPortalContainer(reactKonva.Layer, 'Layer');

exports.Group = Group;
exports.Layer = Layer;
exports.PORTAL_LAYER_ID = PORTAL_LAYER_ID;
exports.Portal = Portal;
exports.Stage = PortalStage;
exports.Z_INDEX = Z_INDEX;
//# sourceMappingURL=index.js.map
